#include "Benchmarker.h"
#include "Application.h"
#include "Logger.h"

#include "OgreCamera.h"
#include "OgreConfigOptionMap.h"
#include "OgreRenderSystem.h"
#include "OgreRenderWindow.h"
#include "OgreRoot.h"


Benchmarker::Benchmarker(Application* application, Ogre::Camera* camera):
	mApplication(application),
	mCamera(camera),
	mTimeInCurrentPhase(0),
	mCurrentPhase(-1),
	mRotationMovement(false),
	mFramesRendered(0),
	mMeasureStartTime(1.0f),
	mMeasureEndTime(6.0f),
	mCamStartPos(-0.2f,  0.5f,  0.1f),
	mCamEndPos	(-0.1f,  0.5f,  0.2f),
	mIsFinished(false),
	mResultText()
{
	Ogre::Root& root = Ogre::Root::getSingleton();
	Ogre::RenderWindow* renderWindow = root.getAutoCreatedWindow();
	mResultText << "\nBENCHMARK\nSystem information:";

	const Ogre::ConfigOptionMap& configOptions = root.getRenderSystem()->getConfigOptions();
	Ogre::ConfigOptionMap::const_iterator iter = configOptions.begin();
	Ogre::ConfigOptionMap::const_iterator end = configOptions.end();
	while (iter != end)
	{
		mResultText
			<< "\n  "
			<< iter->first
			<< ": "
			<< iter->second.currentValue;
		iter++;
	}

	mResultText << "\nResults:\n";

	mNumPhases = 92 + (int)(mApplication->getNumMaterials());

	switchToNextPhase();
}


Benchmarker::~Benchmarker()
{
	if (!mIsFinished)
	{
		std::string notFinishedText = "BENCHMARKING ABORTED, NOT FINISHED!";
		Logger::log(notFinishedText);
		mResultText << "\n" << notFinishedText;
	}
	Logger::log(mResultText.str());
}


void Benchmarker::update(float time)
{
	mTimeInCurrentPhase += time;
	if (mTimeInCurrentPhase >= mMeasureEndTime)
	{
		switchToNextPhase();
	}
	if (mTimeInCurrentPhase >= mMeasureStartTime)
	{
		mFramesRendered++;
	}

	updateCamera();
}


bool Benchmarker::isFinished() const
{
	return mIsFinished;
}


void Benchmarker::updateCamera()
{
	float interpolator = mTimeInCurrentPhase / mMeasureEndTime;
	if (mRotationMovement)
	{
		interpolator *= 6.2832f;
		Ogre::Vector3 rotationCentre(15.5f, 0.5f, -15.5f);
		Ogre::Vector3 localCamPos(sin(interpolator), 0, cos(interpolator));
		mCamera->setPosition(rotationCentre + localCamPos * 22.1f);
		mCamera->lookAt(rotationCentre);
	}
	else
	{
		Ogre::Vector3 camPos = (1 - interpolator) * mCamStartPos + interpolator * mCamEndPos;
		mCamera->setPosition(camPos);
		mCamera->lookAt(camPos + Ogre::Vector3(1, 0, -1));
	}
}


void Benchmarker::switchToNextPhase()
{
	if (mCurrentPhase != -1)
	{
		mResultText << mFramesRendered << "\n";
	}
	mResultText << "  ";

	mTimeInCurrentPhase = 0;
	mCurrentPhase++;
	mFramesRendered = 0;
	std::stringstream text;
	text
		<< "Interior Mapping version 10"
		<< "\nRunning benchmark"
		<< "\nProgress: phase "
		<< mCurrentPhase + 1
		<< " of "
		<< mNumPhases
		<< " ("
		<< mCurrentPhase * mMeasureEndTime
		<< "s of "
		<< mNumPhases * mMeasureEndTime
		<< "s elapsed)"
		<< "\nCurrently testing: ";

	/*
	Phases:
		0 to and including 49:
			even: polygonized with (phase/2) objects
			odd: IM with ((phase-1)/2) objects
		50 to and including 69:
			increasing number of rooms with textures
		70 to and including 89:
			increasing number of rooms without textures
		90 and 91:
			many buildings without and with early z out
		92 to and including (91+numMaterials):
			once with every material
	*/
	if (mCurrentPhase < 50)
	{
		text << "polygonized interiors versus Interior Mapped interiors";

		mResultText << "polygonized versus Interior Mapped\t";
		if (mCurrentPhase % 2 == 0)
		{
			mResultText << "polygonized\t";
			mApplication->switchToMesh(mApplication->getNumMeshes() - 2);
		}
		else
		{
			mResultText << "IM\t";
			mApplication->switchToMesh(mApplication->getNumMeshes() - 1);
		}
		int numObjects = mCurrentPhase / 2 + 1;
		mApplication->setNumObjects(numObjects);
		mResultText << numObjects * numObjects << "\t";
		mRotationMovement = false;
	}
	else if (mCurrentPhase < 70)
	{
		text << "number of walls in the interior, with textures";

		mApplication->setNumObjects(1);
		mApplication->switchToMesh(0);
		mApplication->switchToMaterial(15);
		mRotationMovement = false;
		float phase = (mCurrentPhase - 49) * 10;
		mApplication->setWallFrequences(Ogre::Vector3(phase, phase, phase));

		mResultText
			<< "Frequency of the interior walls (with textures)\t"
			<< phase
			<< "\t1\t";
	}
	else if (mCurrentPhase < 90)
	{
		text << "number of walls in the interior, without textures";

		mApplication->setNumObjects(1);
		mApplication->switchToMesh(0);
		mApplication->switchToMaterial(6);
		mRotationMovement = false;
		float phase = (mCurrentPhase - 69) * 10;
		mApplication->setWallFrequences(Ogre::Vector3(phase, phase, phase));

		mResultText
			<< "Frequency of the interior walls (without textures)\t"
			<< phase
			<< "\t1\t";
	}
	else if (mCurrentPhase < 92)
	{
		text << "early z out on and off";
		mApplication->setNumObjects(1);
		mApplication->switchToMesh(3);
		mRotationMovement = true;
		mResultText << "Early z out\tEarly z-out ";
		if (mCurrentPhase == 90)
		{
			mApplication->switchToMaterial(13);
			mResultText << "off";
		}
		else if (mCurrentPhase == 91)
		{
			mApplication->switchToMaterial(16);
			mResultText << "on";
		}
		mResultText	<< "\t1\t";
	}
	else if (mCurrentPhase < mNumPhases)
	{
		text << "Interior Mapping with different materials";

		mApplication->setNumObjects(1);
		mApplication->switchToMesh(0);
		mApplication->switchToMaterial(mCurrentPhase - 92);
		mRotationMovement = false;

		mResultText
			<< "Different materials\t"
			<< mApplication->getCurrentMaterialName()
			<< "\t1\t";
	}
	else
	{
		mResultText << "Finished benchmarking!";
		mIsFinished = true;
	}

	mApplication->setOverlayTexts(text.str(), "");
}